# PyScatter

PyScatter is a implementation of the scattering transform. It's main
usage is feature extraction from images or one dimensional signals.

## Examples
A basic example of using PyScatter together with scikit-learn can be
found in `scatter_learn_test.py`.

## How to build the tree

```python

from filters.swt import swt2
from filters.helper import OutputAllFilter

# Declare a list of linear filters for each level.
filters = [
    [swt1(wavelet='db1'), someotherfilter(some_arguments)],  # Filters on level 1
    [swt1(wavelet='db1')],  # Filters on level 2
    [OutputAllFilter()]     # Output all leaves of the tree.
]


# Declare nonlinearities.
nonlin = [
            None,   # Let the signal pass. f(x) = x
            np.abs,
            np.real,
        ]
        
# Declare pooling functions.
poolings = [
    None,               # Don't apply any pooling on first level.
    pool(2, np.avg),    # Reduce the signal size by a factor 2.
    pool(2, np.mean),   # Reduce the signal size by a factor 2.
]

# Assemble the tree.
tree = scattering_tree(filters, nonlin, poolings)

transformed = tree.transform(input_signal)
```

## Modules

### scatteringtree
This is the core of the algorithm. The initialized scattering_tree object
can be used as a scikit-learn transform.


### filters
Contains some linear filter sets such as stationary wavelet transforms and Gabor frames.

The scattering tree expects filters to be passed as objects providing the function `apply_filter(input_signal)`.

### nonlinearity

This module contains some default non-linearities.
Nonlinearities must be vectorized functions. Use `np.vectorize` if your function is not yet vectorized.

### pooling

The scattering tree expects pooling functions to be passed as objects providing the function `apply_pooling(input_signal)`.

### visualize
Generates graphical representations of the scattering tree using `graphviz`.

