#!/usr/bin/env python3

import numpy as np

from sklearn import svm
from sklearn.datasets import fetch_mldata
from sklearn.decomposition import PCA, IncrementalPCA
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
from scipy.misc import imresize

#pyscatter libraries
import tree_database
from scatteringtree import scattering_tree
from nonlinearity import *
from pooling import pooling

from scipy.misc import imresize
import cProfile
import os, struct
from array import array as pyarray
from pylab import *

from sklearn.datasets import fetch_mldata

from sklearn.decomposition import PCA, IncrementalPCA
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis

try:
    from joblib import Parallel, delayed
    from multiprocessing import cpu_count
    
    n_jobs = cpu_count()
except:
    n_jobs = 1

from nonlinearity import *



verbose = 8
num_train = 50
num_test = 20



def mnist_to_img(data):
    """bring data into square format with dimension being a power of 2"""
    img_small = data.reshape((28,28))
    return imresize(img_small, (32, 32), interp='bilinear')



def run_test():

    """
    Initialize scattering tree.
    
    Replace 'mixed_tree' by any other struct from tree_database to experiment,
    e.g. 'gabor_tree', 'stationary_wavelet_tree'.
    """
    sct = tree_database.gabor_wavelet_tree

    # # # # # # # # #
    # Training phase.
    #
    
    print('Entering training phase...')
    print('Fetching MNIST dataset...')
    mnist = fetch_mldata('MNIST original', data_home= './mnist_dataset')
    labels = mnist.target
    images = mnist.data

    idx = np.arange(0, int(len(images)/1))
    np.random.seed(0)
    np.random.shuffle(idx)	# We want the test data to be 'randomly' mixed.
    
    idx_training = idx[0:num_train]
    idx_test = idx[num_train:num_train+num_test]

    training_input = images[idx_training]
    training_labels = labels[idx_training]
    
    test_input = images[idx_test]
    test_labels = labels[idx_test]
    
    
    #Convert to 32x32 pixel images to satisfy swt2.
    print('Resample input images...')
    training_input = np.array([mnist_to_img(t) for t in training_input])
    test_input = np.array([mnist_to_img(t) for t in test_input])


    # Apply scattering transform to each training image.
    print('Scattering training data...')
    # Apply scattering transform to each training image.
    if n_jobs > 1:
        scattered_training_output = Parallel(n_jobs=n_jobs, verbose=verbose)(delayed(sct.transform)(inp) for inp in training_input)
    else:
        scattered_training_output = [sct.transform(x) for x in training_input]


    # Dimensionality reduction of training feature vectors (fvs).
    print('Fitting dimensionality reduction...')
    dim_reduce = PCA(n_components=100) # reduce to n_components features
    dim_reduce.fit(scattered_training_output[0:40]) #fitting PCA doesn't benefit from larger training data

    scattered_training_output = dim_reduce.transform(scattered_training_output) # reduce fvs dimensionality


    # Initialize and fit the classifier.
    classifier = svm.SVC(
                         kernel='linear',
                         cache_size=2000,
                         random_state = 0
                         )
    print('Fitting the classifier...')
    classifier.fit(scattered_training_output, training_labels)



    # # # # # # #
    # Test phase.
    #

    print('Scattering test data...')
    # Apply scattering transform to each test image.
    if n_jobs > 1:
        scattered_test_output = Parallel(n_jobs=n_jobs, verbose=verbose)(delayed(sct.transform)(inp) for inp in test_input)
    else:
        scattered_test_output = [sct.transform(x) for x in test_input]

    # Dimensionality reduction of test fvs.
    scattered_test_output = dim_reduce.transform(scattered_test_output)

    # Predict test labels.
    print('Predict test labels...')
    prediction = classifier.predict(scattered_test_output)

    # Analyse prediction accuracy.
    num_correct = np.sum(prediction == test_labels)
    print("prediction accuracy: %.3f%%"%(num_correct/len(prediction)*100))
    
if __name__ == '__main__':
    #cProfile.run('run_test()')
    run_test()
