#!/usr/bin/env python3

"""
This is an example of how the scattering transform looks like if applied to an image.
"""

import numpy as np

from scatteringtree import scattering_tree
from filters.swt import swt2
from filters.helper import OutputAllFilter
from nonlinearity import ReLu
from pooling import pooling

from scipy.misc import imresize

from sklearn.datasets import fetch_mldata

from nonlinearity import *

from visualize import visualize_tree

import tree_database

def mnist_to_img(data):
    img_small = data.reshape((28,28))
    return imresize(img_small, (32, 32), interp='bilinear')


if __name__ == '__main__':
    
    # Download MNIST dataset
    print('fetching MNIST dataset...')
    mnist = fetch_mldata('MNIST original', data_home='./mnist_dataset')
    
    
    # Initialize scattering transform.
    
    # Initialize propagation filters.
    prop_filters = [
                        [swt2(wavelet='db1', levels=1, start_level=0, frequency_decreasing_path=True)],
                        [swt2(wavelet='db1', levels=1, start_level=0, frequency_decreasing_path=True)],
                        [OutputAllFilter()],
                    ]

    
    # Non-linearities.
    nls = [
        ReLu,
        ReLu,
        None,
    ]

    # Pooling functions.
    pools = [
                pooling((2,2), np.max),
                None,
                None,
                ]
    
    # Assemble the scattering transformation.
    sct = scattering_tree(
                            propagation_filters=prop_filters,
                            nonlinearities=nls,
                            poolings=pools
                            )
       
    # Or use a default tree.
    sct = tree_database.gabor_wavelet_tree

    # Resample image to 32x32 pixels.
    data = mnist.data[mnist.target == 5]
    signal = mnist_to_img(data[2])
    
    # Plot the scattering transform.
    visualize_tree(sct, signal)

    
